;********************************************************
;* EE130/230M Course Project
;* Created by: Nuo Xu
;* Version: 04/01/2013
;* Comments: 20nm-Lg N-channel MOSFET Design
;********************************************************

(if (string=? "@Type@" "nMOS") 
 (begin
  (define DopSub "BoronActiveConcentration")
  (define DopSD  "ArsenicActiveConcentration")
  (define SubDop  1.0e15 )   ; [1/cm3]
 )
 (begin 
  (define DopSub "ArsenicActiveConcentration")
  (define DopSD  "BoronActiveConcentration")
  (define SubDop  1.0e15 )   ; [1/cm3]
 )
)

;----------------------------------------------------------------------
; Setting parameters
; - Length for each part
(define Lg    @Lgate@)               ; [um] Gate length
(define Los   0.00)	      ;[um] Extension S/D Spacer length
(define Lsp   @Lsp@)                    ;[um] 
(define Lcont 0.02)		       ; [um] Source/Drain region length
(define Ltot (+ Lg (* 2 Lsp) (* 2 Lcont)) )       ; [um] Total Lateral Dimension 

; - Layer thickness
(define Hsub 0.2)        ; [um] Substrate thickness
(define Tox  @Xo@)    ; [um] Gate oxide thickness
(define Hpol 0.03)         ; [um] Poly gate thickness

; - Junction Depths
(define XjExt  @Xjext@)    ; [um] Source/Drain Extension depth
(define XjSD   0.025)   ; [um] Deep Source/Drain Junction depth

; - others
; - spacer rounding
(define fillet-radius 0.08) ; [um] Rounding radius

; - pn junction resolution
(define Gpn 0.0005)         ; [um]

;----------------------------------------------------------------------
; Derived quantities
(define Xmax (/ Ltot 2.0))
(define Xg   (/ Lg   2.0))
(define Xos  (+ Xg   Los))
(define Xsp  (+ Xg   Lsp))

(define Ysub Hsub)
(define Ygox (* Tox -1.0)) ; Botton of the oxide/Si interface is at Y=0
(define Ypol (- Ygox Hpol))

(define Lcont (- Xmax Xsp))  
;----------------------------------------------------------------------
; Overlap resolution: New replaces Old
(sdegeo:set-default-boolean "ABA")

;----------------------------------------------------------------------
; Creating substrate region
(sdegeo:create-rectangle 
  (position    0.0  Ysub  0.0 ) 
  (position    Xmax 0.0 0.0 ) "Silicon" "R.Substrate" )

; Creating gate oxide
(sdegeo:create-rectangle 
  (position    0.0 0.0  0.0 ) 
  (position    Xg Ygox 0.0 )  "SiO2" "R.Gateox" )

; Creating spacers regions
(sdegeo:create-rectangle 
  (position   Xg  0.0 0.0 ) 
  (position    Xsp  Ypol 0.0 )   "Si3N4" "R.Spacer" )

; Creating PolySi gate
(sdegeo:create-rectangle 
  (position    0.0 Ygox 0.0 ) 
  (position    Xg  Ypol 0.0 )   "PolySi" "R.Polygate" )

;----------------------------------------------------------------------
; - rounding spacer
(sdegeo:fillet-2d 
 (find-vertex-id (position Xsp Ypol 0.0 ))  fillet-radius)

;----------------------------------------------------------------------
; Contact declarations
(sdegeo:define-contact-set "drain" 
  4.0  (color:rgb 0.0 1.0 0.0 ) "##")

(sdegeo:define-contact-set "gate" 
  4.0  (color:rgb 0.0 0.0 1.0 ) "##")

(sdegeo:define-contact-set "substrate"
  4.0  (color:rgb 0.0 1.0 1.0 ) "##")

;----------------------------------------------------------------------
; Contact settings
; for the Drain, find the edge at the middle point between Xmax and Xsp
(sdegeo:define-2d-contact 
 (find-edge-id (position (* (+ Xmax Xsp)  0.5) 0.0 0.0))  "drain")

(define GateMetal (find-body-id (position 5e-4 Ypol 0.0)))
(sdegeo:set-current-contact-set "gate")
(sdegeo:set-contact-boundary-edges GateMetal) 
(sdegeo:delete-region GateMetal) 

(sdegeo:define-2d-contact 
 (find-edge-id (position 5e-4  Ysub 0.0))  "substrate")
 
;----------------------------------------------------------------------
; Separating lumps
(sde:separate-lumps)

; Setting region names
(sde:addmaterial 
  (find-body-id (position  5e-4  (- Ysub 0.1) 0.0)) 
  "Silicon" "R.Substrate")

(sde:addmaterial 
  (find-body-id (position  5e-4  (* 0.5 Ygox) 0.0)) 
  "SiO2"    "R.Gateox")

;(sde:addmaterial 
;  (find-body-id (position  5e-4  (- Ygox 0.01)  0.0)) 
;  "PolySi"  "R.Polygate")

(sde:addmaterial 
  (find-body-id (position (* -0.5 (+ Xsp Xg)) (- Ygox 0.01)  0.0)) 
  "Si3N4"   "R.Spacerleft")

(sde:addmaterial 
  (find-body-id (position (*  0.5 (+ Xsp Xg)) (- Ygox 0.01)  0.0))  
"Si3N4"   "R.Spacerright")

;----------------------------------------------------------------------
; Saving BND file
(sdeio:save-tdr-bnd (get-body-list) "n@node@_half_bnd.tdr")
;----------------------------------------------------------------------

; Profiles:
; - Substrate
(sdedr:define-constant-profile "Const.Substrate" 
 DopSub SubDop )
(sdedr:define-constant-profile-region  "PlaceCD.Substrate" 
 "Const.Substrate" "R.Substrate" )

; - Poly
; (sdedr:define-constant-profile "Const.Gate" 
; DopSD 2.5e20 )
; (sdedr:define-constant-profile-region "PlaceCD.Gate" 
; "Const.Gate" "R.Polygate" )
 
; - Uniform Profile
;  
(sdedr:define-constant-profile "Const.UniDop" DopSub @ChanDoping@ )
(sdedr:define-constant-profile-region  "PlaceCD.UniDop" 
 "Const.UniDop" "R.Substrate" )
 
; - Source/Drain extensions 
; -- base line definitions
(sdedr:define-refinement-window "BaseLine.Ext" "Line"  
 (position    Xos      0.0 0.0)  
 (position (* Xmax 2.0) 0.0 0.0) )
;   implant definition
(sdedr:define-gaussian-profile "Impl.Extprof"
 DopSD "PeakPos" 0  "PeakVal" 8e19 "ValueAtDepth" @ChanDoping@ "Depth" XjExt "Gauss"  "Factor" 0.5 )
; -- implant placement
(sdedr:define-analytical-profile-placement "Impl.Ext" 
 "Impl.Extprof" "BaseLine.Ext" "Positive" "NoReplace" "Eval")

; - Source/Drain implants
; -- base line definitions
(sdedr:define-refinement-window "BaseLine.SD" "Line"  
 (position Xsp 0.0 0.0)  
 (position (* Xmax 2.0) 0.0 0.0) )
; -- implant definition
(sdedr:define-gaussian-profile "Impl.SDprof"
 DopSD 
 "PeakPos" 0  "PeakVal" 1e20
 "ValueAtDepth" @ChanDoping@ "Depth" XjSD "Gauss"  "Factor" 0.5 )
; -- implant placement
(sdedr:define-analytical-profile-placement "Impl.SD" 
 "Impl.SDprof" "BaseLine.SD" "Positive" "NoReplace" "Eval")

;----------------------------------------------------------------------
; Meshing Strategy:
; Substrate
(sdedr:define-refinement-size "Ref.Substrate" 
  (/ Xmax 4.0)  (/ Hsub 8.0) 
  Gpn            Gpn )
(sdedr:define-refinement-function "Ref.Substrate" 
 "DopingConcentration" "MaxTransDiff" 1)
(sdedr:define-refinement-region "RefPlace.Substrate" 
 "Ref.Substrate" "R.Substrate" )

; Active
(sdedr:define-refinement-window "RWin.Act" 
 "Rectangle"  
 (position   0.0  0.0   0.0) 
 (position  Xmax (* XjSD  1.5)    0.0) )
(sdedr:define-refinement-size "Ref.SiAct" 
  (/ Lcont 8.0) (/ XjSD 10.0) 
  Gpn            Gpn )
(sdedr:define-refinement-function "Ref.SiAct" 
 "DopingConcentration" "MaxTransDiff" 1)
(sdedr:define-refinement-placement "RefPlace.SiAct" 
 "Ref.SiAct" "RWin.Act" )

; Po Gate Multibox
; (sdedr:define-refinement-window "MBWindow.Gate" 
; "Rectangle"  
; (position 0.0 Ypol  0.0) 
; (position Xg  Ygox  0.0) )
; (sdedr:define-multibox-size "MBSize.Gate" 
;  (/ Lg  10.0)  (/ Hpol 4.0)
;  (/ Lg 14.0)   2e-4 
;  1.0         -1.35 )
; (sdedr:define-multibox-placement "MBPlace.Gate" 
;  "MBSize.Gate"  "MBWindow.Gate" )

; GateOx Multibox
(sdedr:define-refinement-window "MBWindow.GateOx" 
 "Rectangle"  
 (position 0.0        Ygox  0.0) 
 (position (* Xg  1.2) 0.0  0.0) )
(sdedr:define-multibox-size "MBSize.GateOx" 
  (/ Lg  10.0)  (/ Tox 6.0)
  (/ Lg 12.0)   5e-5 
  1.0         -1.35 )
(sdedr:define-multibox-placement "MBPlace.GateOx" 
 "MBSize.GateOx"  "MBWindow.GateOx" )

; GateOx
(sdedr:define-refinement-size "Ref.GOX" 
  (/ Ltot 8.0)  (/ Tox 8.0) 
  Gpn           (/ Tox 12.0) )
(sdedr:define-refinement-region "RefPlace.GOX" 
 "Ref.GOX" "R.Gateox" )

; Channel Multibox
(sdedr:define-refinement-window "MBWindow.Channel" 
 "Rectangle"  
 (position        0.0  0.0   0.0) 
 (position (* Xg  1.2) (* 2.0 XjExt)  0.0) )
(sdedr:define-multibox-size "MBSize.Channel" 
  (/ Lg  14.0)  (/ XjSD 24.0)
  (/ Lg 18.0)   2e-5 
  1.0           1.35 )
(sdedr:define-multibox-placement "MBPlace.Channel" 
 "MBSize.Channel" "MBWindow.Channel" )

; Junctions under the gate
(sdedr:define-refinement-size "Ref.J" 
  (/ XjExt 24.0)  (/ XjExt 24.0) 
  (/ XjExt 32.0) (/ XjExt 32.0))

; Gate-Drain Junction
(sdedr:define-refinement-window "RWin.GD" 
 "Rectangle"  
 (position (- Xg (* 0.8 XjExt))   0.0   0.0) 
 (position (- Xg (* 0.1 XjExt))   (* XjExt 0.5)  0.0) )
(sdedr:define-refinement-placement "RefPlace.GJ" 
 "Ref.J" "RWin.GD" )

;----------------------------------------------------------------------
; Save CMD file
 (sdedr:write-cmd-file "n@node@_half_msh.cmd")

;----------------------------------------------------------------------
; Build Mesh 
(system:command "snmesh n@node@_half_msh")

;----------------------------------------------------------------------
; Reflect device

(system:command "tdx -mtt -x -ren drain=source n@node@_half_msh n@node@_msh")

